/*
 * Copyright (C) 2009 Chase Douglas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.
 */

#include <arpa/inet.h>
#include <QSocketNotifier>
#include <errno.h>
#include <linux/input.h>
#include <linux/uinput.h>

#include "Client.h"
#include "Input.h"
#include "rinput.h"

Input::Input(Client &_client) :
    client(_client) {
    if (!input.open(stdin, QIODevice::ReadOnly | QIODevice::Text | QIODevice::Unbuffered)) {
        qCritical("Failed to open stdin");
        return;
    }
    if (!output.open(stdout, QIODevice::WriteOnly | QIODevice::Text | QIODevice::Unbuffered)) {
        qCritical("Failed to open stdout");
        return;
    }

    connect(&client, SIGNAL(ready()), SLOT(ready()));
}

void Input::ready() {
    rinput_message_t message;

    message.type = SET_BIT;
    message.data.bit.type = EVBIT;
    message.data.bit.code = EV_KEY;
    hton_rinput(&message);
    client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
    message.type = SET_BIT;
    message.data.bit.type = EVBIT;
    message.data.bit.code = EV_REL;
    hton_rinput(&message);
    client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
    message.type = SET_BIT;
    message.data.bit.type = KEYBIT;
    message.data.bit.code = BTN_MOUSE;
    hton_rinput(&message);
    client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
    message.type = SET_BIT;
    message.data.bit.type = RELBIT;
    message.data.bit.code = REL_X;
    hton_rinput(&message);
    client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
    message.type = SET_BIT;
    message.data.bit.type = RELBIT;
    message.data.bit.code = REL_Y;
    hton_rinput(&message);
    client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
    message.type = CREATE;
    hton_rinput(&message);
    client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));

    output.write("Enter event (btn, x <value>, or y <value>):\n");

    QSocketNotifier *notifier = new QSocketNotifier(0, QSocketNotifier::Read, this);
    connect(notifier, SIGNAL(activated(int)), SLOT(readyRead()));
    notifier->setEnabled(TRUE);
}

void Input::readyRead() {
    QByteArray line(input.readLine());
    line.chop(1);
    QList<QByteArray> args(line.split(' '));

    if (args.at(0) == "btn") {
        rinput_message_t message;
        message.type = EVENT;
        message.data.event.type = EV_KEY;
        message.data.event.code = BTN_MOUSE;
        message.data.event.value = 1;
        hton_rinput(&message);
        client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
        message.type = EVENT;
        message.data.event.type = EV_KEY;
        message.data.event.code = BTN_MOUSE;
        message.data.event.value = 0;
        hton_rinput(&message);
        client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
    }
    else if (args.at(0) == "x") {
        bool ok;
        rinput_message_t message;
        message.data.event.value = args.at(1).toInt(&ok);
        if (!ok) {
            qCritical("Error: Wrong event format");
            return;
        }

        message.type = EVENT;
        message.data.event.type = EV_REL;
        message.data.event.code = REL_X;
        hton_rinput(&message);
        client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
    }
    else if (args.at(0) == "y") {
        bool ok;
        rinput_message_t message;
        message.data.event.value = args.at(1).toInt(&ok);
        if (!ok) {
            qCritical("Error: Wrong event format");
            return;
        }

        message.type = EVENT;
        message.data.event.type = EV_REL;
        message.data.event.code = REL_Y;
        hton_rinput(&message);
        client.write(QByteArray::fromRawData((const char *)&message, sizeof(message)));
    }
    else {
        qCritical("Error: Wrong event format");
    }
}
