/*
 * Copyright (C) 2009 Chase Douglas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.
 */

#include <QtCore/QCoreApplication>
#include <getopt.h>
#include <stdlib.h>

#include "Client.h"
#include "Connection.h"
#include "Input.h"
#include "macros.h"

static void usage(char *name) {
    qCritical(
              "Usage: %s [options]\n"
              "Options:\n"
              "  -u|--user <username>\t\tUsername to authenticate as (required)\n"
              "  -p|--pass <password>\t\tPassword to authenticate with (required)\n"
              "  -t|--target <target>\t\tTarget rinput server (localhost)\n"
              "  -p|--port <port>\t\tPort to connect to rinput server on (8771)\n"
              "  -v|--verbose\t\t\tVerbose output\n"
              "  -q|--quiet\t\t\tQuiet output\n",
              name
             );
}

static void normalMsgHandler(QtMsgType type, const char *msg) {
    switch (type) {
        case QtDebugMsg:
        case QtWarningMsg:
            break;
        case QtCriticalMsg:
            fprintf(stderr, "%s\n", msg);
            break;
        case QtFatalMsg:
            fprintf(stderr, "%s\n", msg);
            abort();
     }
}

static void warningMsgHandler(QtMsgType type, const char *msg) {
    switch (type) {
        case QtDebugMsg:
            break;
        case QtWarningMsg:
            fprintf(stderr, "%s\n", msg);
            break;
        case QtCriticalMsg:
            fprintf(stderr, "%s\n", msg);
            break;
        case QtFatalMsg:
            fprintf(stderr, "%s\n", msg);
            abort();
     }
}

static void debugMsgHandler(QtMsgType type, const char *msg) {
    switch (type) {
        case QtDebugMsg:
            fprintf(stderr, "%s\n", msg);
            break;
        case QtWarningMsg:
            fprintf(stderr, "%s\n", msg);
            break;
        case QtCriticalMsg:
            fprintf(stderr, "%s\n", msg);
            break;
        case QtFatalMsg:
            fprintf(stderr, "%s\n", msg);
            abort();
     }
}

int main(int argc, char *argv[])
{
    QCoreApplication a(argc, argv);

    int verbosity = 1;

    QString target("localhost");
    quint16 port = 8771;

    forever {
        static const struct option longOptions[] = {
            {"help", no_argument, 0, 'h'},
            {"user", required_argument, 0, 'u'},
            {"pass", required_argument, 0, 'p'},
            {"target", required_argument, 0, 't'},
            {"port", required_argument, 0, 'o'},
            {"verbose", no_argument, 0, 'v'},
            {"quiet", no_argument, 0, 'q'},
            {0, 0, 0, 0}
        };

        switch (getopt_long(argc, argv, "hu:p:t:o:vq", longOptions, NULL)) {
            case 'h':
                usage(argv[0]);
                return -1;

            case 'u':
                CONFIG_SET("user", optarg);
                break;

            case 'p':
                CONFIG_SET("pass", optarg);
                break;

            case 't':
                target = optarg;
                break;

            case 'o': {
                QByteArray portString(optarg);
                bool ok;
                port = portString.toUShort(&ok);
                if (!ok) {
                    qCritical("Error: Port number invalid");
                    return -1;
                }
                break;
            }

            case 'v':
                if (verbosity != 1) {
                    qCritical("Error: Verbose and quiet are mutually exclusive and may each only be used once");
                    return -1;
                }
                verbosity++;
                break;

            case 'q':
                if (verbosity != 1) {
                    qCritical("Error: Verbose and quiet are mutually exclusive and may each only be used once");
                    return -1;
                }
                verbosity--;
                break;

            case -1:
                goto done;

            case '?':
                usage(argv[0]);
                return -1;

            default:
                qFatal("getopt_long failed to return a valid value");
                break;
        }
    }

done:
    switch(verbosity) {
        case 0:
            qInstallMsgHandler(normalMsgHandler);
            break;
        case 1:
            qInstallMsgHandler(warningMsgHandler);
            break;
        default:
            qInstallMsgHandler(debugMsgHandler);
            break;
    }

    if (!CONFIG_VALID("user")) {
        qCritical("Error: Username not provided");
    }
    if (!CONFIG_VALID("pass")) {
        qCritical("Error: Password not provided");
    }
    if (!CONFIG_VALID("user") || !CONFIG_VALID("pass")) {
        return -1;
    }
    if (CONFIG("user").toString().length() + CONFIG("pass").toString().length() > 252) {
        qCritical("Error: Username and password must be less than 252 combined characters");
        return -1;
    }

    Client client(target, port);
    Input input(client);

    QObject::connect(&client, SIGNAL(quit()), &a, SLOT(quit()));

    return a.exec();
}
